#coding:utf-8
######################################################
# Author: Chen KX <ckx1025ckx@gmail.com>             #
# License: BSD 2-Clause                              #
######################################################

import ida_graph
import idc
import ida_kernwin
import idaapi

def get_bb_id(graph, ea):
    for block in graph:
        if block.start_ea <= ea and block.end_ea > ea:
            return block.id

def reset_all_block_color_to_white(addr):
    white = 0xffffff
    f = idaapi.get_func(addr)
    g = idaapi.FlowChart(f, flags=idaapi.FC_PREDS) 
    p = idaapi.node_info_t()
    p.bg_color = white
    base_block_ea = f.start_ea
    for block in g:
        idaapi.set_node_info(
            base_block_ea,
            block.id, 
            p,
            idaapi.NIF_BG_COLOR | idaapi.NIF_FRAME_COLOR)


class ckxTraceRender(object):
    def __init__(self):
        self.widget_a = ida_kernwin.find_widget("IDA View-A")
        if not self.widget_a:
            raise Exception("IDA View-A not opened!")
        self.current_block_start_ea=None
        self.current_focused_addr = None
        self.current_bb_id = None
        self.current_viewer = None
    def get_current_viewer(self):
        self.current_viewer=ida_kernwin.get_current_viewer()
        print("get current_viewer")
    def setAddr(self,address):
        	
        if self.current_focused_addr == address:
            return
        else:
            idc.jumpto(address)
        f = idaapi.get_func(address)
        # print(f)
        if f == None:
            print("not found function address...")
            return
        g = idaapi.FlowChart(f, flags=idaapi.FC_PREDS) 
        target_bb_id = get_bb_id(g, address)
        # print(target_bb_id)
        
        if self.current_block_start_ea==f.start_ea and self.current_bb_id == target_bb_id:
            return
        else:

            self.recoverColor()
            self.current_block_start_ea = f.start_ea
            self.current_focused_addr = address
            self.current_bb_id = target_bb_id
            self.changeColor()
            ida_graph.refresh_viewer(self.widget_a)

    def changeColor(self):
        p = idaapi.node_info_t()
        color =0xffff00
        p.bg_color = color
        idaapi.set_node_info(
            self.current_block_start_ea,
            self.current_bb_id, 
            p,
            idaapi.NIF_BG_COLOR | idaapi.NIF_FRAME_COLOR)
        

    def recoverColor(self):
        if not self.current_focused_addr:
            # print("1")
            return
        if self.current_bb_id ==None:
            # print("2")
            return
        p = idaapi.node_info_t()
        color =0xffffff
        p.bg_color = color
        idaapi.set_node_info(
            self.current_block_start_ea, 
            self.current_bb_id, 
            p,
            idaapi.NIF_BG_COLOR | idaapi.NIF_FRAME_COLOR)
        # print("3")
        # ida_graph.refresh_viewer(self.widget_a)


ckxTR = ckxTraceRender()
# ckxTR.setAddr(0x40C9A0)
# ckxTR.setAddr(0x40C9E2)

# ckxTR.setAddr(0x60)
# ckxTR.setAddr(0x69)
# ckxTR.setAddr(0x4a)
# reset_all_block_color_to_white(0x4a)




# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'ckxtraceform.ui'
#
# Created by: PyQt5 UI code generator 5.12.3
#
# WARNING! All changes made in this file will be lost!


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_ckxTraceViewObj(object):
    def setupUi(self, ckxTraceViewObj):
        ckxTraceViewObj.setObjectName("ckxTraceViewObj")
        ckxTraceViewObj.resize(256, 391)
        self.listWidget = QtWidgets.QListWidget(ckxTraceViewObj)
        self.listWidget.setGeometry(QtCore.QRect(30, 20, 201, 221))
        self.listWidget.setObjectName("listWidget")
        self.pushButton_fileexp = QtWidgets.QPushButton(ckxTraceViewObj)
        self.pushButton_fileexp.setGeometry(QtCore.QRect(20, 340, 101, 31))
        self.pushButton_fileexp.setObjectName("pushButton_fileexp")
        self.pushButton_loattrace = QtWidgets.QPushButton(ckxTraceViewObj)
        self.pushButton_loattrace.setGeometry(QtCore.QRect(120, 340, 101, 31))
        self.pushButton_loattrace.setObjectName("pushButton_loattrace")
        self.pushButton_searchnext = QtWidgets.QPushButton(ckxTraceViewObj)
        self.pushButton_searchnext.setEnabled(False)
        self.pushButton_searchnext.setGeometry(QtCore.QRect(210, 270, 31, 31))
        self.pushButton_searchnext.setObjectName("pushButton_searchnext")
        self.widget = QtWidgets.QWidget(ckxTraceViewObj)
        self.widget.setGeometry(QtCore.QRect(29, 249, 208, 20))
        self.widget.setObjectName("widget")
        self.horizontalLayout = QtWidgets.QHBoxLayout(self.widget)
        self.horizontalLayout.setContentsMargins(0, 0, 0, 0)
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.radioButton_manual = QtWidgets.QRadioButton(self.widget)
        self.radioButton_manual.setChecked(True)
        self.radioButton_manual.setObjectName("radioButton_manual")
        self.horizontalLayout.addWidget(self.radioButton_manual)
        self.radioButton_auto = QtWidgets.QRadioButton(self.widget)
        self.radioButton_auto.setObjectName("radioButton_auto")
        self.horizontalLayout.addWidget(self.radioButton_auto)
        self.widget1 = QtWidgets.QWidget(ckxTraceViewObj)
        self.widget1.setGeometry(QtCore.QRect(30, 280, 171, 21))
        self.widget1.setObjectName("widget1")
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout(self.widget1)
        self.horizontalLayout_2.setContentsMargins(0, 0, 0, 0)
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.label = QtWidgets.QLabel(self.widget1)
        self.label.setObjectName("label")
        self.horizontalLayout_2.addWidget(self.label)
        self.lineEdit_search = QtWidgets.QLineEdit(self.widget1)
        self.lineEdit_search.setObjectName("lineEdit_search")
        self.horizontalLayout_2.addWidget(self.lineEdit_search)
        self.widget2 = QtWidgets.QWidget(ckxTraceViewObj)
        self.widget2.setGeometry(QtCore.QRect(30, 310, 194, 23))
        self.widget2.setObjectName("widget2")
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout(self.widget2)
        self.horizontalLayout_3.setContentsMargins(0, 0, 0, 0)
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.label_2 = QtWidgets.QLabel(self.widget2)
        self.label_2.setObjectName("label_2")
        self.horizontalLayout_3.addWidget(self.label_2)
        self.lineEdit_tracefile = QtWidgets.QLineEdit(self.widget2)
        self.lineEdit_tracefile.setObjectName("lineEdit_tracefile")
        self.horizontalLayout_3.addWidget(self.lineEdit_tracefile)

        self.retranslateUi(ckxTraceViewObj)
        self.pushButton_loattrace.clicked.connect(ckxTraceViewObj.ckx_loadtrace)
        self.pushButton_fileexp.clicked.connect(ckxTraceViewObj.ckx_filechoose)
        self.listWidget.itemSelectionChanged.connect(ckxTraceViewObj.ckx_onSelectionChanged)
        self.listWidget.itemClicked['QListWidgetItem*'].connect(ckxTraceViewObj.ckx_onItemClicked)
        self.lineEdit_search.returnPressed.connect(ckxTraceViewObj.ckx_search_address)
        self.pushButton_searchnext.clicked.connect(ckxTraceViewObj.ckx_next_button_clicked)
        QtCore.QMetaObject.connectSlotsByName(ckxTraceViewObj)

    def retranslateUi(self, ckxTraceViewObj):
        _translate = QtCore.QCoreApplication.translate
        ckxTraceViewObj.setWindowTitle(_translate("ckxTraceViewObj", "Form"))
        self.pushButton_fileexp.setText(_translate("ckxTraceViewObj", "File Explore"))
        self.pushButton_loattrace.setText(_translate("ckxTraceViewObj", "Load Trace"))
        self.pushButton_searchnext.setText(_translate("ckxTraceViewObj", ">"))
        self.radioButton_manual.setText(_translate("ckxTraceViewObj", "Manual Mode"))
        self.radioButton_auto.setText(_translate("ckxTraceViewObj", "Auto Trace"))
        self.label.setText(_translate("ckxTraceViewObj", "Search 0x"))
        self.label_2.setText(_translate("ckxTraceViewObj", "TraceFile:"))



#coding:utf-8
######################################################
# Author: Chen KX <ckx1025ckx@gmail.com>             #
# License: BSD 2-Clause                              #
######################################################

from idaapi import PluginForm
from PyQt5 import QtCore, QtGui
from PyQt5 import QtWidgets
from PyQt5.QtWidgets import QPushButton
import sip
from .mainlogic import ckxTraceViewPanel


class MyPluginFormClass(PluginForm):
    def OnCreate(self, form):
        """
        Called when the plugin form is created
        """
        # Get parent widget
        # print("......")
        # print(type(form))
        self.parent = self.FormToPyQtWidget(form)
        self.PopulateForm()
        
    # def ckx(self):
        # print("ckx.....")
    def PopulateForm(self):
        # Create layout
        self.tvPannel=ckxTraceViewPanel(self.parent)
        print("....|||")
        self.tvPannel.show()
        # tvPannel = Ui_Form()
        # tvPannel.setupUi(self.parent)

    def OnClose(self, form):
        self.tvPannel.onClose()
        """
        Called when the plugin form is closed
        """
        pass
    def Show(self):
        # 调用顺序是先show，然后再onCreate的，所以我们这里不能瞎重载
        # print("showinng ....")
        return super(MyPluginFormClass,self).Show("ckxTraceView",options = (PluginForm.WOPN_TAB | PluginForm.WCLS_CLOSE_LATER))






import sys
from PyQt5.QtGui import *
from PyQt5.QtCore import *
from PyQt5.QtWidgets import *
from ckxtraceviewer.mainlogic import ckxTraceViewPanel


app = QApplication(sys.argv)
tv = ckxTraceViewPanel()
tv.show()
app.exec_()
import sys
sys.exit()



# -*- encoding: utf8 -*-
######################################################
# Author: Chen KX <ckx1025ckx@gmail.com>             #
# License: BSD 2-Clause                              #
######################################################


from setuptools import setup

long_description = \
"""
ckxtraceviewer
======
ckxtraceviewer is a python-only solution to add a trace view to IDA Pro. 
Use `<Ctrl-Alt-I>` to open a window with an embedded _Qt console_. 
You can then load an execution trace which are automatically synchronized with `IDA VIEW-A`.
See full README on GitHub: <https://www.github.com/ckx/ckxtraceviewer>.
"""

setup(name='ckxtraceviewer',
      version='1.0',
      description='IDA plugin to embed a trace view inside IDA',
      long_description=long_description,
      long_description_content_type="text/markdown",
      author='Chen KX',
      author_email='ckx1025ckx@gmail.com',
      url='https://www.github.com/ckx/ckxtraceviewer',
      packages=['ckxtraceviewer','ckxtraceviewer/ui'],
      install_requires=[
          'monkeyhex>=1.0',
      ],
      license="BSD",
      classifiers=[
          "Development Status :: 5 - Production/Stable",
          "Environment :: Plugins",
          "License :: OSI Approved :: BSD License",
          "Programming Language :: Python :: 2.7",
          "Programming Language :: Python :: 3",
      ],
)


#coding:utf-8
######################################################
# Author: Chen KX <ckx1025ckx@gmail.com>             #
# License: BSD 2-Clause                              #
######################################################

import sys
from os import path
# from PyQt5
from PyQt5.QtGui import *
from PyQt5.QtCore import *
from PyQt5.QtWidgets import *
import queue
try:
    from .ui.ckxtraceform import Ui_ckxTraceViewObj
    from .ckxrender_in_ida import ckxTR
    import ida_kernwin
except:
    try:
        from ui.ckxtraceform import Ui_ckxTraceViewObj
    except:
        from .ui.ckxtraceform import Ui_ckxTraceViewObj
    pass


default_filepath="/Users/ckx/myproject/qtuis/testpyqt/tmp.trace"

def load_file_to_list(alist,filepath):
    num = 1
    f=open(filepath,"r")
    aline=f.readline()
    while aline:
        aline = aline.split("\n")[0]
        # aline
        QListWidgetItem(aline,alist)
        aline=f.readline()
        if num%1000==0:
            print(num)
        num=num+1

def set_focus_in_ida(address_int):
    print("operations in ida")
    try:
    # if True:
        # if ckxTR.current_viewer == None:
            # ckxTR.get_current_viewer()
        ckxTR.setAddr(address_int)
        # ida_kernwin.activate_widget(ckxTR.current_viewer,True)
        # self.listWidget.setFocus()
    except:
        print("not in real ida")
    pass


class ckxTraceViewPanel(QWidget,Ui_ckxTraceViewObj):
    def __init__(self,parent=None):
        if not parent:
            super(ckxTraceViewPanel,self).__init__(parent)
            self.setupUi(self)
        else:
            super(ckxTraceViewPanel,self).__init__(parent)
            self.setupUi(self)
        self.lineEdit_tracefile.setText(default_filepath)
        self.radioButton_manual.setChecked(True)
        self.is_trace_loaded=False
    def onClose(self):
        try:
            ckxTR.recoverColor()
            ida_graph.refresh_viewer(ckxTR.widget_a)
        except:
            pass
        pass
    def ckx_loadtrace(self):
        if self.is_trace_loaded:
            self.listWidget.clear()
            # print("cleared")
        def check_fileline_valid(fileline):
            if path.exists(fileline):
                return True
            else:
                return False
            
            pass
            return True
        fileline = self.lineEdit_tracefile.text()
        if not check_fileline_valid(fileline):
            self.lineEdit_tracefile.setText("invalid fileline:"+fileline)
            return
        alist=self.listWidget
        load_file_to_list(alist,fileline)
        self.is_trace_loaded=True
        pass
    def ckx_onSelectionChanged(self):
        if self.radioButton_auto.isChecked():
            print("onSelectionChanged do sth....")
            it=self.listWidget.currentItem()
            print(it.text())
            aline = it.text()
            try:
                addr=int(aline,16)
            except ValueError:
                print("not a valid hex address")
                return
            set_focus_in_ida(addr)
            self.listWidget.setFocus()

        else:
            return
        pass
    def ckx_onItemClicked(self,theitem):
        if self.radioButton_manual.isChecked():
            print(theitem.text())
            aline = theitem.text()
            try:
                addr=int(aline,16)
            except ValueError:
                print("not a valid hex address")
                return
            set_focus_in_ida(addr)
            self.listWidget.setFocus()
        else:
            return
    def ckx_search_address(self):
        self.pushButton_searchnext.setEnabled(False)
        self.searchresult = None
        aline = self.lineEdit_search.text()
        try:
            addr= int(aline,16)
        except ValueError:
            print("search : not a valid hex address")
            return
        # print(hex(addr))
        # target = hex(addr)
        target = aline
        resultlist=self.listWidget.findItems(aline,Qt.MatchFlag.MatchExactly)
        if len(resultlist) ==0:
            print("not found matched!")
        else:
            allnum = len(resultlist)
            if allnum>1:
                self.pushButton_searchnext.setEnabled(True)
            self.searchresult= queue.Queue()
            for i in resultlist:
                self.searchresult.put(i)
            # self.searchresult = resultlist
            cur_item = self.searchresult.get()
            self.listWidget.setCurrentItem(cur_item)
            self.searchresult.put(cur_item)

    def ckx_next_button_clicked(self):
        if self.searchresult is not None:
            cur_item = self.searchresult.get()
            self.listWidget.setCurrentItem(cur_item)
            self.searchresult.put(cur_item)
        pass
    def ckx_filechoose(self):
        if False:
            # for debugging usage
            import ipdb
            ipdb.set_trace()
        if True:
            fname = QFileDialog.getOpenFileName(self, 'Open file', '~/','All Files (*)')[0]
            self.lineEdit_tracefile.setText(fname)
        pass
        print("Todo: file choose")
    
    
